
import ajenti
from ajenti.api import *
from ajenti.plugins import *

info = PluginInfo(
    title = 'GIT',
    icon = 'folder-close',
    description='Git Management Tool with Gitolite3 Authorization',
    dependencies =
    [
        PluginDependency('main'),
        PluginDependency('services'),
        BinaryDependency('git'),
        BinaryDependency('gitolite')
    ]
)

def init():
    import main;


from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Firewall',
    description='Linux iptables firewall',
    icon='fire',
    dependencies=[
        PluginDependency('main')
    ],
)


def init():
    import main



import subprocess

from ajenti.api import plugin
from ajenti.plugins.dashboard.api import DashboardWidget
from ajenti.plugins.dashboard.api import ConfigurableWidget


@plugin
class MyUnameWidget(ConfigurableWidget):
    name = _('uname')
    icon = 'cog'
    options = 'asnrvmpio'

    def on_prepare(self):
        self.append(self.ui.inflate('uname:widget'))

    def on_start(self):
        args = ''
        if self.config['opt_a']:
            args += 'a'
        else:
            for o in self.options[1:]:
                if self.config['opt_%s' % o]:
                    args += o

        if len(args):
            args = '-' + args
        self.find('value').text = subprocess.check_output(['uname', args])

    def create_config(self):
        options = {}
        for o in self.options:
            options['opt_%s'%o] = o in 'srm'
        return options

    def on_config_start(self):
        for o in self.options:
            opt = 'opt_%s' % o
            self.dialog.find(opt).value = self.config[opt]

    def on_config_save(self):
        for o in self.options:
            opt = 'opt_%s' % o
            self.config[opt] = self.dialog.find(opt).value



import os
import subprocess
import re

from pwd import getpwnam

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

class Repository (object):
    def __init__(self, name):
        self.name = name
        self.rwusers = ''
        self.rousers = '@all'

class User (object):
    def __init__(self, name, pubkey):
        self.name = name
        self.pubkey = pubkey

#class Group (object):
#    def __init__(self, name):
#        self.name = name
#        self.members = []


@plugin
class GitPlugin (SectionPlugin):
    def init(self):
        self.title = _('Git')
        self.icon = 'folder-close'
        self.category = 'Software'
        self.append(self.ui.inflate('git:main'))
        self.binder = Binder(None, self)
        self.user = None

        # Find User
        for user in ['gitolite3', 'gitolite2', 'gitolite']:
             try:
                 pwn = getpwnam(user)
                 self.user = pwn
                 break
             except:
                 pass

        if self.user == None:
            exit(-1)

        # Set Conf & Key Directory
        self.confdir = os.path.join(self.user.pw_dir, '.gitolite/conf/')
        self.keydir = os.path.join(self.user.pw_dir, '.gitolite/keydir/')

        # Event for new items
        self.find('usercollection').new_item = lambda c: User('newuser', '')
        self.find('repocollection').new_item = lambda c: Repository('newrepo')

        # Remove old Gitolite.conf and replace our version
        os.remove(self.confdir + 'gitolite.conf')
        with open((self.confdir + 'gitolite.conf'), 'w+') as file:
            file.write('include "*.conf"')

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.refresh_users()
        self.refresh_repositories()
        self.binder.setup(self).populate()

    def reload(self):
        subprocess.call('su -c "gitolite compile" ' + self.user.pw_name, shell=True)
        self.context.notify('info', _('Saved'))

    #
    # Repositories
    #
    def refresh_repositories(self):
        self.repositories = []

        userlist = []
        for user in self.users:
            userlist.append(user.name)
        self.find('rwusers').values = userlist
        self.find('rwusers').labels = userlist
        self.find('rousers').values = userlist
        self.find('rousers').labels = userlist

        for file in os.listdir(self.confdir):
            if os.path.isfile(self.confdir + file) and not 'gitolite.conf' in file:
                with open(self.confdir + file) as content:
                    content = content.read()
                    repo = Repository('')

                    # Repository Name
                    match = re.search(r'repo[\s]+([A-z0-9]+)', content, re.I)
                    repo.name = match.group(1)

                    #Repository Users(read-write)
                    match = re.search(r'RW\+[\s]+\=[\s]+([^\n]+)', content, re.I)
                    repo.rwusers = match is not None and match.group(1) or ''

                    #Repository Users(read-only)
                    match = re.search(r'R[\s]+\=[\s]+([^\n]+)', content, re.I)
                    repo.rousers = match is not None and match.group(1) or ''

                    self.repositories.append(repo)

    @on('save-repositories', 'click')
    def save_repositories(self):
        self.binder.update()

        files = []
        for file in os.listdir(self.confdir):
            if os.path.isfile(self.confdir + file) and not "gitolite.conf" in file:
                files.append(file)

        for repo in self.repositories:
            with open((self.confdir + repo.name + '.conf'), 'w+') as file:
                content = 'repo '
                content += repo.name
                content += '\n'

                rwusers = repo.rwusers.strip()
                if len(rwusers) > 0:
                    content += '\tRW+\t=\t' + rwusers + '\n'

                rousers = repo.rousers.strip()
                if len(rousers) > 0:
                    content += '\tR\t=\t' + rousers + '\n'

                file.write(content)

            if (repo.name + '.conf') in files:
                files.remove(repo.name + '.conf')

        for file in files:
            os.remove(self.confdir + file)

        self.reload()

    #
    # Users
    #
    def refresh_users(self):
        self.users = []

        for file in os.listdir(self.keydir):
            with open(self.keydir + file) as content:
                username = file.replace('.pub', '')
                pubkey = content.read()

                self.users.append(User(username, pubkey))

    @on('save-users', 'click')
    def save_users(self):
        self.binder.update()

        files = []
        for file in os.listdir(self.keydir):
            files.append(file)

        for user in self.users:
            with open((self.keydir + user.name + '.pub'), 'w+') as file:
                file.write(user.pubkey)
            if (user.name + '.pub') in files:
                files.remove(user.name + '.pub')

        for file in files:
            os.remove(self.keydir + file)

        self.reload();

#    #
#    # Groups
#    #
#    def refresh_groups(self):
#        self.groups = []
#
#    @on('save-groups', 'click')
#    def save_groups(self):
#        self.binder.update()
#        #
#        # ...
#        #
#        self.reload();







import re
from datetime import datetime
import os
import time
import subprocess

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.plugins.services.api import ServiceMultiplexor
from ajenti.ui import on
from ajenti.ui.binder import Binder
from ajenti.util import platform_select

from reconfigure.configs.base import Reconfig
from reconfigure.parsers import SSVParser
from reconfigure.builders import BoundBuilder
from reconfigure.nodes import Node, PropertyNode
from reconfigure.items.bound import BoundData

open_ntpd_conf = '/etc/openntpd/ntpd.conf'
ntpd_conf = '/etc/ntp.conf'


def isopenntpd():
    return not ServiceMultiplexor.get().get_one('openntpd') is None


class NTPDData(BoundData):
    pass


class ServerData(BoundData):
    def template(self):
        return Node(
            'line',
            Node('token', PropertyNode('value', 'server')),
            Node('token', PropertyNode('value', '0.pool.ntp.org')),
        )


NTPDData.bind_collection('servers', selector=lambda x: x.children[0].get('value').value == 'server',
                         item_class=ServerData)
ServerData.bind_property('value', 'address', path=lambda x: x.children[1])


class NTPDConfig(Reconfig):
    def __init__(self, **kwargs):
        k = {
            'parser': SSVParser(),
            'builder': BoundBuilder(NTPDData),
        }
        k.update(kwargs)
        Reconfig.__init__(self, **k)


@plugin
class NTPDPlugin(SectionPlugin):
    def get_tz_debian(self):
        return open('/etc/timezone').read().strip()

    def get_tz_nondebian(self):
        return os.path.realpath('/etc/localtime')[len('/usr/share/zoneinfo/'):] if os.path.islink(
            '/etc/localtime') else ''

    def set_tz_debian(self, timezone):
        open('/etc/timezone', 'w').write(timezone + '\n')

    def set_tz_nondebian(self, timezone):
        tz = os.path.join('/usr/share/zoneinfo/', timezone)
        if os.path.exists('/etc/localtime'):
            os.unlink('/etc/localtime')
        os.symlink(tz, '/etc/localtime')

    openntpd = isopenntpd()

    service_name = platform_select(
        default='openntpd' if openntpd else 'ntp',
        centos='openntpd' if openntpd else 'ntpd',
        mageia='ntpd',
    )

    get_tz = platform_select(
        debian=get_tz_debian,
        default=get_tz_nondebian,
    )

    set_tz = platform_select(
        debian=set_tz_debian,
        default=set_tz_nondebian,
    )

    def init(self):
        self.title = _('Date & Time')
        self.icon = 'time'
        self.category = _('System')

        self.append(self.ui.inflate('ntpd:main'))

        self.find('servicebar').name = self.service_name
        self.find('servicebar').reload()

        self.config = NTPDConfig(path=platform_select(
            default=open_ntpd_conf if self.openntpd else ntpd_conf,
            centos='/usr/local/etc/ntpd.conf' if self.openntpd else ntpd_conf,
            freebsd='/usr/local/etc/ntpd.conf' if self.openntpd else ntpd_conf,
        ))

        self.binder = Binder(None, self)

        self.available_zones = []
        for d, dirs, files in os.walk('/usr/share/zoneinfo', followlinks=False):
            for f in files:
                if f != 'zone.tab':
                    self.available_zones.append(os.path.join(d, f))
        self.available_zones = [x[len('/usr/share/zoneinfo/'):] for x in self.available_zones]
        self.available_zones.sort()

        self.find('servers').new_item = lambda c: ServerData()

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.config.load()
        self.now = int(time.time())
        self.timezone = self.get_tz()
        self.binder.setup(self).populate()

    @on('set', 'click')
    def on_set(self):
        self.binder.update()
        d = datetime.fromtimestamp(self.now)
        s = d.strftime('%m%d%H%M%Y')
        self.set_tz(self.timezone)
        subprocess.call(['date', s])
        self.refresh()

    @on('sync', 'click')
    def on_sync(self):
        self.binder.update()
        if len(self.config.tree.servers) == 0:
            self.context.notify('error', _('No servers defined'))
            return
        server = self.config.tree.servers[0].address
        output = subprocess.check_output(['ntpdate', '-u', server])
        self.context.notify('info', _('Done'))
        self.context.notify('info', output)
        self.refresh()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        self.config.save()
        self.refresh()
        self.context.notify('info', _('Saved'))
        ServiceMultiplexor.get().get_one(self.service_name).restart()



import subprocess

from ajenti.api import *
from api import PackageInfo, PackageManager


@plugin
@rootcontext
@persistent
class UrpmiPackageManager (PackageManager):
    platforms = ['mageia']
    def refresh(self):
        try:
            out_update = subprocess.check_output(['urpmi.update', '-a'])
        except subprocess.CalledProcessError as e:
            pass

        try:
            out_u = subprocess.check_output(['urpmq', '--auto-select', "-r"])
            self.upgradeable = self._parse_urpmi(out_u)
        except subprocess.CalledProcessError as e:
            self.upgradeable = []

        try:
            out_a = subprocess.check_output(['urpmq', '--list', "-r"])
            self.all = self._parse_urpmi(out_a)
            self.all_dict = dict((x.name, x) for x in self.all)
        except subprocess.CalledProcessError as e:
            self.all = []
            self.all_dict = {}

    def search(self, query):
        if query.strip() == "":
            return []
        try:
            out_s = subprocess.check_output([
                'urpmq', "-Y",
                "--summary", query
            ])
        except subprocess.CalledProcessError as e:
            return []
        
        r = []
        for l in out_s.split('\n'):
            s = l.strip().split(":", 1)
            if len(s) != 2:
                continue
            summary_list = s[1].rsplit("(",1)
            if len(summary_list) != 2:
                continue
            p = PackageInfo()
            p.name = s[0].strip()
            p.summary = summary_list[0].strip()
            p.version = summary_list[1].rsplit(")",1)[0].strip()
            p.state = 'r'
            if p.name in self.all_dict and self.all_dict[p.name].state == 'i':
                p.state = 'i'
            r.append(p)
        return r
    def get_lists(self):
        self.context.launch(
            'terminal',
            command='urpmi.update -a ; read -p "Press [enter] to continue"'
        )

    def do(self, actions, callback=lambda: 0):
        to_install = [a for a in actions if a.action == 'i']
        to_remove = [a for a in actions if a.action == 'r']
        cmd = ''
        if len(to_install) > 0:
            cmd += 'urpmi ' + ' '.join(a.name for a in to_install)
            if len(to_remove) > 0:
                cmd += ' && '
        if len(to_remove) > 0:
            cmd += 'urpme ' + ' '.join(a.name for a in to_remove)
        if len(to_install) > 0 or len(to_remove) > 0:
            cmd += " ; "
        cmd += 'read -p "Press [enter] to continue"'
        self.context.launch('terminal', command=cmd, callback=callback)

    def _parse_urpmi(self, ss):
        r = []
        for s in ss.splitlines():
            s = s.strip().rsplit("-", 2)
            if len(s) != 3:
                continue
            try:
                p = PackageInfo()
                p.name = s[0]
                p.version = "-".join(s[1:])
                p.state = 'i'
                r.append(p)
                if len(r.keys()) > 2500:
                    break
            except:
                pass
        return r



import logging
import os
import rethinkdb as r
import subprocess
import tempfile

from ajenti.api import *
from ajenti.plugins.configurator.api import ClassConfigEditor
from ajenti.plugins.db_common.api import Database, User


@plugin
class RethinkClassConfigEditor (ClassConfigEditor):
    title = 'RethinkDB'
    icon = 'table'

    def init(self):
        self.append(self.ui.inflate('rethink:config'))


class RethinkConnection (object):
    def __init__(self, db):
        self.client = r.connect(host=db.classconfig['RETHINKDB_HOST'],
                         port=db.classconfig['RETHINKDB_PORT'],
                         auth_key=str(db.classconfig['RETHINKDB_AUTH']))

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.client.close()


@plugin
class RethinkDB (BasePlugin):
    default_classconfig = { 
                            'RETHINKDB_HOST': 'localhost',
                            'RETHINKDB_PORT': '28015',
                            'RETHINKDB_AUTH': ''
                        }
    classconfig_editor = RethinkClassConfigEditor

    def connect(self):
        return RethinkConnection(self)

    def query_sql(self, db, sql):
        with self.connect() as rethink:
            #FIXME: Need security for sql
            result =  eval('r.db(\'%s\').%s.run(rethink.client)'%(db,sql))
            return [[x] for x in str(result).splitlines()]

    def query_databases(self):
        result = []
        with self.connect() as rethink:
            for x in r.db_list().run(rethink.client):
                d = Database()
                d.name = x
                result.append(d)
        return result

    def query_drop(self, db):
        with self.connect() as rethink:
            return r.db_drop(db).run(rethink.client)

    def query_create(self, name):
        with self.connect() as rethink:
            return r.db_create(name).run(rethink.client)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='RethinkDB',
    icon='table',
    dependencies=[
        PluginDependency('db_common'),
        BinaryDependency('rethinkdb'),
        ModuleDependency('rethinkdb'),
    ],
)


def init():
    import api
    import main
